; Count.ASM
;
; PIC Counting Demonstration
; Uses three push buttons to count "UP", "Down" and also to "Reset"
; We use LSB MID & MSB regs as digit counters,
; and the counting is done using BCD arithmetic.
; ---------------------------------------------------------------------------

Count           bsf     STATUS,RP0              ; Select page 1
                movlw   B'00000011'             ; set all PORTA inputs to digital
                movwf   ADCON0                  ; (actually this is OPTION reg in page 1)
                bcf     STATUS,RP0              ; back to page 0

                call    ResetCount              ; clear counters

; display Heading

                call    Counter_S               ; display "Counter"

; display Counters

Count00         movlw   D'10'                   ; point to LCD
                call    Position
                call    Dec_Out
                call    DeBounce                 ; wait for all buttons released

; test the buttons

Count01         btfsc   PORTA,0                 ; is "Up" pressed?
                goto    Count02                 ; no.
                call    IncCount                ; yes, increment counters
                goto    Count00                 ; and re-display

Count02         btfsc   PORTA,1                 ; is "Down" pressed?
                goto    Count03                 ; no.
                call    DecCount                ; yes, decrement counters
                goto    Count00                 ; and re-display

Count03         btfsc   PORTA,2                 ; is "Reset" pressed?
                goto    Count01                 ; no.
                call    ResetCount              ; yes, reset counters
                goto    Count00                 ; and re-display

; ---------------------------------------------------------------------------
; Increment Counters:

IncCount        incf    LSD,F                   ; increment the first digit
                movlw   D'10'                   ; did it roll past "09" ?
                subwf   LSD,W
                btfss   STATUS,Z
                return

                clrf    LSD
                incf    MID,F                   ; increment the second digit
                movlw   D'10'                   ; did it roll past "09" ?
                subwf   MID,W
                btfss   STATUS,Z
                return

                clrf    MID
                incf    MSD,F                   ; increment the third digit
                movlw   D'10'                   ; did it roll past "09" ?
                subwf   MSD,W
                btfss   STATUS,Z
                return

                clrf    MID                     ; reset past '999'
                clrf    LSD
                clrf    MSD
                return
                           
; ---------------------------------------------------------------------------
; Decrement Counters:

DecCount        decf    LSD,F                   ; increment the first digit
                incfsz  LSD,W                   ; did it roll past Zero ?
                return

                movlw   D'09'
                movwf   LSD
                decf    MID,F                   ; increment the second digit
                incfsz  MID,W                   ; did it roll past Zero ?
                return

                movlw   D'09'
                movwf   MID
                decf    MSD,F                   ; increment the third digit
                incfsz  MSD,W                   ; did it roll past Zero ?
                return

                movlw   D'09'
                movwf   MSD
                return

; ---------------------------------------------------------------------------
; Reset Counters:

ResetCount      clrf    LSD                  ; clear counters
                clrf    MID
                clrf    MSD
                return

; ---------------------------------------------------------------------------
; Wait for buttons to be released: Mechanical buttons exhibit "contact bounce"
; (or chatter) which micros are fast enough to mistake for genuine button pushes,
; so to be certain, we test twice with a time delay in between.

DeBounce        call    Release                 ; are any buttons down?
                movlw   D'10'                   ; no
                call    Delay                   ; but wait a bit
                call    Release                 ; then check again
                return

Release         btfss   PORTA,0                 ; is "Up" relesaed?
                goto    Release                 ; no, wait.
                btfss   PORTA,1                 ; is "Down" released?
                goto    Release                 ; no, wait.
                btfss   PORTA,2                 ; is "Clear" released?
                goto    Release                 ; no, wait.
                return

