LIST P=16C71
;                          Multi.ASM
;
;               Microchip PIC Multi-Function Demonstration PCB
;
; Copyright     DIY Electronics
;               PO Box 88458
;               Sham Shui PO
;               Hong Hong

; Revision      V1.00    10/8/95    (original release)
;               V1.01    29/9/95     Fahrenheit conversion added

;-------------------------------------------------------------------
;       Port A,  Bit 0 = I/O to external plug
;                Bit 1 = I/O to external plug
;                Bit 2 = I/O to external plug
;                Bit 3 = I/O to external plug
;                Bit 4 = I/O to external plug  (resistor pulls high)
;
;       Port B,  Bit 0 = output, RS line to LCD Display
;                    1 = output, EN line to LCD
;                    2 = I/O to External plug
;                    3 = I/O to External plug
;                    4 = output, LSB data to LCD
;                    5 = output,     data to LCD
;                    6 = output,     data to LCD
;                    7 = output, MSB data to LCD
;
;-------------------------------------------------------------------
                Radix DEC

     include "c:\mpasm\p16cxx.inc"       ; standard PIC definition file
     __CONFIG _CP_OFF & _WDT_OFF & _XT_OSC & _PWRTE_ON   ; set fuses
                                                                
; define RAM register usage

                ORG     H'0C'   ; internal RAM starts here
                
Temp1           RES     1       ; used by Temperature routine
Temp2           RES     1       ; ,,   ,,      ,,
TempComp        RES     1       ; used by Humidity routine
Pointer         RES     1       ; ,,   ,,      ,,
Humidity        RES     1       ; ,,   ,,      ,,
        
TimerFlag       RES     1       ; used by Timer routine
LCDtemp         RES     1       ; used by "LCD"

LSD             RES     1       ; Right hand digit
MID             RES     1       ; Middle
MSD             RES     1       ; Left digit

Low_Count       RES     1       ; Delay in LCD
High_Count      RES     1       ; ,,    ,,

ACCbLO          RES     1       ; Used by double presision maths
ACCbHI          RES     1

Timer_mSec      RES     1       ; Used by Stop watch
Timer_Sec       RES     1
Timer_Min       RES     1

SaveStatus      RES     1       ; Used by interupt routine
SaveWReg        RES     1
                                        
; -------------------------------------------------------

        ORG     H'00'                   ; setup up interupt and reset vectors
	goto    Start
        ORG     H'04'
        goto    Interrupt

Start           movlw   B'11111111'             ; Port A data
                movwf   PORTA
                movlw   B'00000000'             ; Port B data
                movwf   PORTB

                bsf     STATUS,RP0              ; Select page 1
                movlw   B'00000111'             ;  Pullups enabled (bit 7 = 0)
                movwf   TMR0                    ; (actually OPTION reg in page 1)

                movlw   B'11111111'             ; data direction Port A
                movwf   PORTA
                movlw   B'00001100'             ; data direction Port B
                movwf   PORTB
                bcf     STATUS,RP0              ; back to page 0 of RAM

                movlw   H'FF'                   ; wait for LCD to settle down
                call    Delay
                call    InitLCD                 ; initialise LCD
                call    Blank

; Test to see which board is plugged in

                btfsc   PORTB,3                 ; Is it A/D board?
                goto    Start00                 ; no.
                goto    AtoD                    ; yes, run A/D routine.

Start00         btfsc   PORTB,2                 ; Is it Timer/StopWatch board?
                goto    Start01                 ; no.
                btfsc   PORTA,4                 ; yes, where is the switch set?
                goto    Count                   ; is Counter
                goto    Timer                   ; is Timer

Start01         btfsc   PORTA,4                 ; Is it the Temp/Humid board?
                goto    Start10                 ; no
                                                ; yes, but first select digital I/O
                                                ; so we can read "F/C" switch setting
                bsf     STATUS,RP0              ; Select page 1
                movlw   B'00000011'             ; select digital I/O
                movwf   ADCON0                  ; (actually ADCON1 reg in page 1)
                bcf     STATUS,RP0              ; back to page 0 of RAM
                btfsc   PORTA,0                 ; is switch set to "F" or "C" ?
                goto    MeasureTempF            ; run Farhenheight routines
                goto    MeasureTempC            ; run Centigrade routines

Start10         call    DIY_S                   ; no boards plugged in, so say hello
Stop            goto    Stop                    ; and stop here.

;---------------------------------------------------------------------------------------------------------
; Note we have split each major function into a separate file
; to make them easier to understand and re-use.
;
                include "LCD.asm"                ; the LCD drivers
                include "A_D.asm"                ; Analogue to Digital converter
                include "Count.asm"              ; Up/Down counter
                include "Timer.asm"              ; Timer/StopWatch routines
                include "Strings.asm"            ; Various Ascii strings
                include "Temperat.asm"           ; Temp/Humid routines

;---------------------------------------------------------------------------------------------------------

	END
                
