; Timer.ASM
;
; PIC Timing Demonstration
; Uses three push buttons to "START", "STOP" and "Reset" the timer.
; ---------------------------------------------------------------------------------------------------------

Timer           bsf     STATUS,RP0              ; Select page 1
                movlw   B'00000011'             ; set all PORTA inputs to digital
                movwf   ADCON0                  ; (actually this is OPTION reg in page 1)
                movlw   B'00000111'             ; 0 - PS0 = 1   Prescaler divide by 256
                                                ; 1 - PS0 = 1
                                                ; 2 - PS0 = 1
                                                ; 3 - PSA = 0   Prescaler belongs to Timer
                                                ; 4 - RTE = 0
                                                ; 5 - RTS = 0   internal clock
                                                ; 6 - INTEDG = 0
                                                ; 7 - RBPL = 0   Pullups disabled
                movwf   TMR0                    ; (actually OPTION reg in page 1)
                bcf     STATUS,RP0              ; back to page 0

                call    ResetTime               ; clear counters
                call    SWatch_S

; display Timers

Timer00         call    Disp_MinSec
                clrf    TimerFlag               ; used as "count has changed" flag
                                                ; (so we only update the display when necessary)
; test the buttons

Timer01         btfsc   PORTA,0                 ; is "Start" pressed?
                goto    Timer02                 ; no.
                call    StartTime               ; yes, Start timer
                goto    Timer00                 ; and re-display

Timer02         btfsc   PORTA,1                 ; is "Stop" pressed?
                goto    Timer03                 ; no.
                call    StopTime                ; yes, Stop timer
                goto    Timer00                 ; and re-display

Timer03         btfss   PORTA,2                 ; is "Reset" pressed?
                call    ResetTime               ; yes, reset timer
                btfsc   TimerFlag,0             ; has display changed?
                goto    Timer00                 ; yes, so re-display
                goto    Timer01                 ; no.

; ---------------------------------------------------------------------------------------------------------
; Timing module Interrupt Service Routine
;
; The TMR0 Interrupt is used to generate timing pulses.
; With a 4.194304 MhZ clock, this gives 1.048576 internally.
; The prescaler is set to divide by 256 and the TMR0 also by 256.
; With a 4 MhZ clock this gives 16 Hz "timer ticks".
; A further division of 15 gives 1 Sec increments.

Interrupt       movwf   SaveWReg                ; save regs
                swapf   STATUS,W                ; (affects no STATUS bits)
                movwf   SaveStatus              ; save STATUS Reg

                decfsz  Timer_mSec,F             ; has time reached zero?
                goto    RestoreInt              ; no, abandon
                movlw   D'16'                   ; yes, reset mSec counter
                movwf   Timer_mSec
                bsf     TimerFlag,0            ; flag for "count has changed"

                incf    Timer_Sec,F
                movlw   D'60'                   ; have Seconds reached 60 ?
                subwf   Timer_Sec,W
                btfss   STATUS,Z
                goto    RestoreInt              ; no, skip
                clrf    Timer_Sec               ; yes, reset

                incf    Timer_Min,F

RestoreInt      swapf   SaveStatus,W
                movwf   STATUS                  ; restore STATUS Reg
                swapf   SaveWReg,F               ; save WREG
                swapf   SaveWReg,W              ; restore WREG
                bcf     INTCON,T0IF             ; clear the interupt
                retfie
                
;------------------------------------------------------------
; Display Mins and Seconds eg. "123:34"

Disp_MinSec     movlw   .10
                call    Position
                movf    Timer_Min,W
                call    BinBCD
                movf    MSD,W
                call    Ascii
                movf    MID,W
                call    Ascii
                movf    LSD,W
                call    Ascii
                movlw   .58                ; ":"
                call    Display
                movf    Timer_Sec,W
                call    BinBCD
                movf    MID,W
                call    Ascii
                movf    LSD,W
                call    Ascii
                return

; ---------------------------------------------------------------------------
; Start Timer:

StartTime       bsf     INTCON,GIE              ; Enable interupts
                bsf     INTCON,T0IE
                return
                               
; ---------------------------------------------------------------------------
; Stop Timer:

StopTime        bcf     INTCON,GIE              ; Disable interupts
                bcf     INTCON,T0IE
                return

; ---------------------------------------------------------------------------
; Reset Timer:

ResetTime       bcf     INTCON,GIE              ; Disable interupts
                bcf     INTCON,T0IE
                movlw   D'16'
                movwf   Timer_mSec              ; reset timers
                clrf    Timer_Sec
                clrf    Timer_Min
                bsf     TimerFlag,0            ; flag for "count changed"
                return

